@@ -11,6 +11,7 @@ from os import path, environ
 from mingus.core import diatonic
 import Options
 import Movements
+import math
 
 APP_NAME = "qtImproviser"
 APP_VERSION = "0.8"
@@ -26,6 +27,187 @@ class OptionClass:
 	verbose=True
 	meter = (4,4)
 
+class MovementScene(QtGui.QGraphicsScene):
+
+	IOFFSET = 50
+	BOXSIZE = 25
+
+	def __init__(self, main):
+		QtGui.QGraphicsScene.__init__(self)
+		self.main = main
+		self.ui = main.ui
+
+		# Brushes and pens
+		self.brush = QtGui.QBrush(QtCore.Qt.SolidPattern)
+		self.brush_dont_play = QtGui.QBrush(QtCore.Qt.white, QtCore.Qt.NoBrush)
+		self.brush_play = QtGui.QBrush(QtCore.Qt.red, QtCore.Qt.SolidPattern)
+		self.brush_percussion = QtGui.QBrush(QtCore.Qt.yellow, QtCore.Qt.SolidPattern)
+		self.brush_selection = QtGui.QBrush(QtGui.QColor(0, 0, 255).light(186), QtCore.Qt.SolidPattern)
+		self.box_pen = QtGui.QPen(self.brush, 2, QtCore.Qt.SolidLine, QtCore.Qt.RoundCap, QtCore.Qt.RoundJoin)
+		self.select_pen = QtGui.QPen(self.brush, 1, QtCore.Qt.SolidLine, QtCore.Qt.RoundCap, QtCore.Qt.RoundJoin)
+
+	def mousePressEvent(self, ev):
+		self.pressed = ev.scenePos()
+
+	def mouseReleaseEvent(self, ev):
+		if self.pressed == ev.scenePos():
+			x, y = self.get_box_coords(ev)
+			if y >= 0:
+				self.ui.instruments.setCurrentRow(y)
+				self.update()
+			if x >= 0 and y >= 0:
+				print x, y
+
+	def get_box_coords(self, ev):
+		x, y = self.pressed.x(), self.pressed.y()
+		x = x - 200
+		y = y - self.IOFFSET
+		boxx = x / self.BOXSIZE
+		boxy = y / self.BOXSIZE
+		x = -1
+		y = -1
+		if boxy - math.floor(boxy) < 0.8:
+			y = int(math.floor(boxy))
+		if boxx - math.floor(boxx) < 0.8:
+			x = int(math.floor(boxx))
+		return (x, y)
+
+	def mouseDoubleClickEvent(self, ev):
+		if self.pressed == ev.scenePos():
+			x, y = self.get_box_coords(ev)
+			if x >= 0 and y >= 0:
+				self.ui.editinstrument.click()
+		
+		self.pressed = []
+
+	def plays(self, params, n):
+		start, end, step = -1, -1, -1
+		if 'start' in params:
+			start = params['start']
+		if 'step' in params:
+			step = params['step']
+		if 'end' in params:
+			end = params['end']
+
+		if start == -1:
+			return True
+		if n < start:
+			return False
+
+		if end == -1 and step == -1:
+			return True
+
+		if step == -1:
+			if n >= start and n < end:
+				return True
+			return False
+
+		if end == -1:
+			n = (n - start) % (step * 2)
+			if n < step:
+				return True
+			return False
+
+		n = (n - start) % (end - start + step)
+		if n < end - start:
+			return True
+		return False
+
+	def get_progressions(self):
+		try:
+			prog = [Options.parse_progression(x) for x in self.main.get_progressions().split(",")]
+			prog_text = [ x.split()[0] for x in self.main.get_progressions().split(",")]
+			return (prog, prog_text)
+		except:
+			return ([], [])
+
+	def get_prog_block_index(self, prog):
+		duration = int(self.ui.duration.value())
+		prog_block_index, blocks, prog_index, offset = [], [], 0, 0
+		if self.ui.blocks.count() > 0 and len(prog) != 0:
+			for x in self.main.get_blocks().split(","):
+				parts = x.split()
+				params = Options.parse_block_params(parts[1:])
+				dur = duration
+				if 'duration' in params:
+					dur = params['duration']
+				for i in range(dur):
+					prog_block_index.append((offset, parts[0], prog_index))
+					prog_offset = 0
+					for itk, tk in enumerate(prog[prog_index]):
+						if itk != 0:
+							offset += (tk[0] - prog_offset) * len(prog[prog_index][itk - 1][1])
+						prog_offset = tk[0]
+				prog_index += 1
+				if prog_index >= len(prog):
+					prog_index =0
+		return (offset, prog_block_index)
+
+	def paint_prog_block_index(self):
+		prog, prog_text = self.get_progressions()
+		end, prog_block_index = self.get_prog_block_index(prog)
+		IOFFSET = self.IOFFSET
+		BOXSIZE = self.BOXSIZE
+
+		if len(prog) <= 0 or self.main.get_instruments() is None:
+			return end
+
+		for i, x in enumerate(prog_block_index):
+			offset, name, prog_index = x
+			blockend = end
+			if i != len(prog_block_index) - 1:
+				blockend = prog_block_index[i + 1][0]
+			if name != 'R':
+				self.addLine(QtCore.QLineF(200 + offset * BOXSIZE, IOFFSET - 10, 
+					195 + blockend * BOXSIZE, IOFFSET - 10), self.box_pen)
+				self.addLine(QtCore.QLineF(200 + offset * BOXSIZE, IOFFSET - 10, 
+					200 + offset * BOXSIZE, IOFFSET - 8), self.box_pen)
+				self.addLine(QtCore.QLineF(195 + blockend * BOXSIZE, IOFFSET - 10, 
+					195 + blockend * BOXSIZE, IOFFSET - 8), self.box_pen)
+				t = self.addText(name[:int((blockend-offset) * 2.5)], 
+					QtGui.QFont("", 12, 80))
+				t.translate(200 + BOXSIZE * offset, 3)
+				if blockend - offset > 2:
+					t = self.addText(prog_text[prog_index], QtGui.QFont("", 8, 40, True))
+					t.translate(200 + BOXSIZE * offset, IOFFSET - 25)
+		return end
+
+
+	def update(self):
+		IOFFSET = self.IOFFSET
+		BOXSIZE = self.BOXSIZE
+			
+		for x in self.items():
+			self.removeItem(x)
+		
+		end = self.paint_prog_block_index()
+	
+
+		sel = self.ui.instruments.currentRow()
+		if sel != -1:
+			r = self.addRect(QtCore.QRectF(-1, sel * BOXSIZE + IOFFSET - 1, self.width(), BOXSIZE - BOXSIZE / 5 + 1), self.select_pen, self.brush_selection)
+			r.setZValue(-50)
+
+		instr = self.main.get_instruments()
+		if instr is not None:
+			for i, x in enumerate(instr.split(",")):
+				parts = x.split()
+				name = parts[0]
+				params = Options.parse_instrument_params(parts[1:])
+				s = self.addText(name)
+				s.translate(0, i * BOXSIZE + IOFFSET)
+				for n in range(end + 1):
+					if self.plays(params, n):
+						b = self.brush_play
+						if 'channel' in params:
+							if params['channel'] == 9:
+								b = self.brush_percussion
+					else:
+						b = self.brush_dont_play
+					a = self.addRect(QtCore.QRectF(200 + n * BOXSIZE, i * BOXSIZE + IOFFSET, BOXSIZE - BOXSIZE / 5, BOXSIZE - BOXSIZE / 5 ), self.box_pen,b)
+
+		self.ui.graphicsView.setScene(self)
+
 
 class ImproviserMainWindow(QtGui.QMainWindow):
 
@@ -35,9 +217,9 @@ class ImproviserMainWindow(QtGui.QMainWindow):
 
 	def __init__(self):
 		QtGui.QMainWindow.__init__(self)
-
 		self.ui = Ui_MainWindow()
 		self.ui.setupUi(self)
+		self.scene = MovementScene(self)
 		self.fill_combos()
 		self.set_defaults()
 
@@ -59,7 +241,13 @@ class ImproviserMainWindow(QtGui.QMainWindow):
 			self.ui.visualization.addItem(x)
 				
 
+	def wresize(self):
+		print "resize"
+
 	def connect_widgets(self):
+		self.connect(self,
+			QtCore.SIGNAL("resizeEvent(event)"),
+			lambda x: self.wresize())
 		self.connect(self.ui.startbutton, 
 			QtCore.SIGNAL("clicked()"), 
 			self.start_simulation)
@@ -203,7 +391,11 @@ class ImproviserMainWindow(QtGui.QMainWindow):
 			self.ui.menuUpload.setEnabled(True)
 		else:
 			self.ui.menuUpload.setEnabled(False)
-		
+
+		self.update_scene()
+
+	def update_scene(self):
+		self.scene.update()
 
 	def highlighted_movement(self, index):
 		m = getattr(Movements, str(self.ui.movement.itemText(index)))
@@ -468,18 +660,21 @@ class ImproviserMainWindow(QtGui.QMainWindow):
 
 
 	def progression_changed(self):
+		self.update_scene()
 		if self.ui.progressions.count() == 0:
 			self.enable_progression(False)
 		else:
 			self.enable_progression(True)
 
 	def instrument_changed(self):
+		self.update_scene()
 		if self.ui.instruments.count() == 0:
 			self.enable_instruments(False)
 		else:
 			self.enable_instruments(True)
 		
 	def block_changed(self):
+		self.update_scene()
 		if self.ui.blocks.count() == 0:
 			self.enable_blocks(False)
 		else:
@@ -586,6 +781,7 @@ class ImproviserMainWindow(QtGui.QMainWindow):
 		p.load_progression(cur.text())
 		p.show()
 		p.exec_()
+		self.progression_changed()
 
 
 	def get_file_name(self, name, n = 0):
@@ -787,6 +983,12 @@ class ImproviserMainWindow(QtGui.QMainWindow):
 		t = i2.text()
 		i2.setText(i1)
 		cur.setText(t)
+		if lst == self.ui.instruments:
+			self.instrument_changed()
+		elif lst == self.ui.progressions:
+			self.progression_changed()
+		elif lst == self.ui.blocks:
+			self.block_changed()
 
 				
 
